import paho.mqtt.client as mqtt
import datetime
import pytz
import json
import base64
import pandas as pd
import configparser
import logging
import class_files_util

class init():

    def __init__(self, filePath):
        self.filePath = filePath
        self.handleCFG = configparser.ConfigParser()

    def parseCFGData(self):

        self.handleCFG.read(self.filePath)  
        mqttBroker = self.handleCFG['DEFAULT']['MQTT_BROKER']
        seaDeepMQTTPubTopic = self.handleCFG['DEFAULT']['SEA_DEEP_MQTT_PUB_TOPIC']
        iaMQTTSubTopic = self.handleCFG['DEFAULT']['IA_MQTT_SUB_TOPIC']
        portID = self.handleCFG['DEFAULT']['MQTT_PORT']
        mqttSession = self.handleCFG['DEFAULT']['MQTT_SESSION']
        iaLogPath = self.handleCFG['DEFAULT']['IA_LOG_PATH']
        mqttLogPath = self.handleCFG['DEFAULT']['MQTT_LOG_PATH']
        iaTimeZone = self.handleCFG['DEFAULT']['IA_TIME_ZONE']
        iaTenantID = self.handleCFG['IA PROFILE']['TENANT_ID']
        iaUUID = self.handleCFG['IA PROFILE']['UUID']
        iaDevModelNo = self.handleCFG['IA PROFILE']['DEV_MODEL_NO']
        iaLocation = self.handleCFG['IA PROFILE']['LOCATION']
        iaStatus = self.handleCFG['IA PROFILE']['STATUS']
        iaInitializaDateTime = self.handleCFG['IA PROFILE']['INIT_DATETIME']
        iaHBInterval = self.handleCFG['IA PROFILE']['HB_INTERVAL']
        iaInboundPath = self.handleCFG['IA PROFILE']['INBOUND_PATH']
        iaOutboundPath = self.handleCFG['IA PROFILE']['OUTBOUND_PATH']
        modelDeployClassesProfile = self.handleCFG['AI MODEL']['MODEL_DEPLOY_CLASSES_PROFILE']
        modelClassPath = self.handleCFG['AI MODEL']['MODEL_CLASS_PATH']
        modelTaskID = self.handleCFG['AI MODEL']['MODEL_TASK_ID']
        modelWeightPath = self.handleCFG['AI MODEL']['MODEL_WEIGHT_PATH']
        modelFactorPath = self.handleCFG['AI MODEL']['MODEL_FACTOR_PATH']
        modelInfClassPath = self.handleCFG['AI MODEL']['MODEL_INF_CLASS_PATH']
        modelDeployDateTime = self.handleCFG['AI MODEL']['MODEL_DEPLOY_DATETIME']

        parseCFGData = {
                    'mqtt_broker': mqttBroker,
                    'sea_deep_mqtt_pub_topic': seaDeepMQTTPubTopic,
                    'ia_mqtt_sub_topic': iaMQTTSubTopic,
                    'mqtt_port': portID,
                    'mqtt_session': mqttSession,
                    'ia_log_path': iaLogPath,
                    'mqtt_log_path': mqttLogPath,
                    'ia_time_zone': iaTimeZone,
                    'ia_uuid': iaUUID,
                    'ia_tenant_id': iaTenantID,
                    'ia_location': iaLocation,
                    'ia_dev_model_no': iaDevModelNo,
                    'ia_status': iaStatus,
                    'ia_init_date_time': iaInitializaDateTime,
                    'ia_hb_interval': iaHBInterval,
                    'ia_inbound_path': iaInboundPath,
                    'ia_outbound_path': iaOutboundPath,
                    'model_deploy_classes_profile': modelDeployClassesProfile,
                    'model_class_path': modelClassPath,
                    'model_task_id': modelTaskID,
                    'model_weight_path': modelWeightPath,
                    'model_factor_path': modelFactorPath,
                    'model_inf_class_path': modelInfClassPath,
                    'model_deploy_datetime': modelDeployDateTime
                  }

        return(parseCFGData)
    
    # purpose: to update the configuration file w/ specified section and parameter
    def writeData(self, sectionName, parName, parValue):
        self.handleCFG.read(self.filePath)
        self.handleCFG.set(sectionName, parName, parValue)
        with open(self.filePath, 'w') as configFile:
            self.handleCFG.write(configFile)
        return       

    def updateCfgCurrentStatus(self, iaNewID):

        #print('update IA[%s] ia.ini status.'%(iaNewID))

        cfgData = self.parseCFGData()
        iaIDString = cfgData['ia_uuid']
        iaStatusString = cfgData['ia_status']
        iaInitDateTimeString = cfgData['ia_init_date_time']
        iaTimeZone = cfgData['ia_time_zone']
        #iaNewInitDateTime = datetime.datetime.now(pytz.timezone('Asia/Taipei')).strftime("%Y-%m-%d %H:%M:%S")       
        iaNewInitDateTime = datetime.datetime.now(pytz.timezone(iaTimeZone)).strftime("%Y-%m-%d %H:%M:%S")       
        #print('iaIDString=======>', iaIDString)
        if iaIDString=='':
            # iaStatus:
            # init: the 1st start/launch for IA device and ready to deployment
            # ready: after IA subscription finished
            # deployed: AI model deployed
            # blocked: IA been blocked from IA administrator
            # hold: IA been blocked from SEA DEEP server send request
            iaNewStatus = 'init'
            self.writeData('IA PROFILE', 'UUID', iaNewID)
            self.writeData('IA PROFILE', 'STATUS', iaNewStatus)
            self.writeData('IA PROFILE', 'INIT_DATETIME', iaNewInitDateTime)
        else:
            iaIDLists = iaIDString.split(',')
            iaStatusLists = iaStatusString.split(',')
            iaInitDateTimeLists = iaInitDateTimeString.split(',')
            iaNewStatus = self.getStatusByID(iaNewID, iaIDLists, iaStatusLists)
            if iaNewStatus=='':
                iaNewStatus='init'
                iaNewIDString = iaIDString + ',' + iaNewID
                iaNewStatusString = iaStatusString + ',' + iaNewStatus
                iaNewInitDateTimeString = iaInitDateTimeString + ',' + iaNewInitDateTime
            else:
                if  iaNewStatus == 'init':
                    iaNewStatus = 'ready'
                    iaNewIDString = iaIDString
                    iaNewStatusString = self.updateStringToken(iaIDLists, iaNewID, iaStatusLists, iaNewStatus)
                    iaNewInitDateTimeString = iaInitDateTimeString
                else:
                    iaNewIDString = iaIDString
                    iaNewStatusString = iaStatusString
                    #iaNewInitDateTimeString = self.updateStringToken(iaIDLists, iaNewID, iaInitDateTimeLists, iaNewInitDateTime)
                    iaNewInitDateTimeString = iaInitDateTimeString

            self.writeData('IA PROFILE', 'UUID', iaNewIDString)
            self.writeData('IA PROFILE', 'STATUS', iaNewStatusString)
            self.writeData('IA PROFILE', 'INIT_DATETIME', iaNewInitDateTimeString)

    def getCurrentStatus(self, iaNewID):

        # get the ia.init configuration data        
        cfgData = self.parseCFGData()
        iaIDLists = cfgData['ia_uuid'].split(',')
        iaStatusLists = cfgData['ia_status'].split(',')
        iaDateTimeLists = cfgData['ia_init_date_time'].split(',')
        iaModelDeployClassesProfile = cfgData['model_deploy_classes_profile'].split(',')
        iaNewStatus = self.getStatusByID(iaNewID, iaIDLists, iaStatusLists)
        iaNewInitDateTime = self.getStatusByID(iaNewID, iaIDLists, iaDateTimeLists)
        if iaModelDeployClassesProfile != '':
            modelDownloadStatus = 'YES'
        else:
            modelDownloadStatus = 'NO'

        payloadMessage = json.dumps({
                        'send_tmsp': datetime.datetime.now(pytz.timezone('Asia/Taipei')).strftime("%Y-%m-%d %H:%M:%S"),
                        'mqtt_type': 'ia_heartbeat', 
                        'ia_tenant_id':cfgData['ia_tenant_id'], 
                        'ia_id':iaNewID, 
                        'location': cfgData['ia_location'],
                        'dev_model_no': cfgData['ia_dev_model_no'],
                        'ia_status': iaNewStatus,
                        'ia_time_zone': cfgData['ia_time_zone'],
                        'ia_hb_interval': cfgData['ia_hb_interval'],
                        'init_datetime': iaNewInitDateTime,
                        'model_deploy_classes_profile': cfgData['model_deploy_classes_profile'],
                        'model_class_path': cfgData['model_class_path'],
                        'model_task_id': cfgData['model_task_id'],
                        'model_weight_path': cfgData['model_weight_path'],
                        'model_deploy_datetime': cfgData['model_deploy_datetime'],
                        'model_download_status': modelDownloadStatus
                        })

        return payloadMessage

    def msg_log(self, logMsg):
        iaLogFilePath = '.' + self.parseCFGData()['ia_log_path'] + '/ia_log.log'
        logging.basicConfig(
            format = '%(asctime)s %(levelname)s %(message)s', 
            datefmt = '%Y-%b-%d %H:%M:%S', 
            filename= iaLogFilePath, 
            level = logging.INFO)
        logging.info('IA launch: [%s]'%(logMsg))
   
    def msg_queue(self, payloadMsg):
        fileName = datetime.datetime.now().strftime("%Y%m%d-%H%M%S.%f") + '.raw'
        print('write file to [%s]!'%(fileName))
        with open('.' + self.parseCFGData()['ia_inbound_path'] + '/' + fileName, 'wb') as writer:
            writer.write(payloadMsg)
        return

    def getStatusByID(self, iaNewID, iaIDLists, iaStatusLists):
        idx=-1
        for i in range(0, len(iaIDLists)):
            if iaNewID==iaIDLists[i]:
                idx=i
                break
        if idx==-1:
            return ''
        else:
            return iaStatusLists[idx]

    def updateStringToken(self, iaIDLists, iaID, dataLists, updateToken):
        idx=0
        for i in range(0, len(iaIDLists)):
            if iaID==iaIDLists[i]:
                idx=i
                break
        tokenString = ''
        for i in range(0, len(dataLists)):
            if i==idx:
                tokenString = tokenString + ',' + updateToken
            else:
                tokenString = tokenString + ',' + dataLists[i]
        tokenString = tokenString.lstrip(',')
        print('tokenString====>', tokenString)
        return tokenString
    
    def updateModelTaskProfile(self, iaID, taskID, className, weightFile, factorFile, infClassFile, deployDateTime):

        taskString = self.parseCFGData()['model_task_id']
        if taskString=='':
            self.writeData('AI MODEL', 'MODEL_TASK_ID', taskID)            
        else:
            if taskID not in taskString:
                taskString = taskString + ',' + taskID
                self.writeData('AI MODEL', 'MODEL_TASK_ID', taskID)            
        
        modelProfileString = self.parseCFGData()['model_deploy_classes_profile']
        modelProfileData=json.dumps({
            'class_name': className,
            'weight_file': weightFile,
            'factor_file': factorFile,
            'inf_class_file': infClassFile,
            'deploy_date_time': deployDateTime
        })
        if modelProfileString=='':
            self.writeData('AI MODEL', 'MODEL_DEPLOY_CLASSES_PROFILE', modelProfileData)            
        else:
            if modelProfileData not in modelProfileString:
                modelProfileString = modelProfileString + ',' + modelProfileData    
                self.writeData('AI MODEL', 'MODEL_DEPLOY_CLASSES_PROFILE', modelProfileString)         

        iaIDLists = self.parseCFGData()['ia_uuid'].split(',')
        dataLists = self.parseCFGData()['ia_status'].split(',')
        updateToken = 'deployed'
        iaStatusString = self.updateStringToken(iaIDLists, iaID, dataLists, updateToken)
        self.writeData('IA PROFILE', 'STATUS', iaStatusString)